/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Authors :
 *  Milan Crha <mcrha@redhat.com>
 *
 * Copyright © 2013 Red Hat, inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301
 * USA
 */

#ifndef E_EWS_GNOME_3_8_COMPAT_H
#define E_EWS_GNOME_3_8_COMPAT_H

#include <libsoup/soup.h>
#include <gio/gio.h>
#include <camel/camel.h>

void		g_simple_async_result_take_error(GSimpleAsyncResult *result,
						 GError *error);
void		g_simple_async_result_set_check_cancellable
						(GSimpleAsyncResult *simple,
						 GCancellable *check_cancellable);

const gchar *	soup_uri_get_host		(SoupURI *suri);
const gchar *	soup_uri_get_scheme		(SoupURI *suri);

void		g_mutex_init			(GMutex **mutex);
void		g_mutex_clear			(GMutex **mutex);

#define GRecMutex GStaticRecMutex
#define g_rec_mutex_init g_static_rec_mutex_init
#define g_rec_mutex_clear g_static_rec_mutex_free
#define g_rec_mutex_lock g_static_rec_mutex_lock
#define g_rec_mutex_unlock g_static_rec_mutex_unlock

gchar *		e_util_strdup_strip		(const gchar *string);

typedef struct _EAsyncClosure EAsyncClosure;

EAsyncClosure *	e_async_closure_new		(void);
GAsyncResult *	e_async_closure_wait		(EAsyncClosure *closure);
void		e_async_closure_free		(EAsyncClosure *closure);
void		e_async_closure_callback	(GObject *object,
						 GAsyncResult *result,
						 gpointer closure);

void		g_slist_free_full		(GSList *slist,
						 GDestroyNotify destroy_func);
void		g_list_free_full		(GList *list,
						 GDestroyNotify destroy_func);

const gchar *	camel_service_get_user_data_dir	(CamelService *service);
const gchar *	camel_service_get_user_cache_dir(CamelService *service);

int		g_access			(const gchar *filename,
						 int mode);
gboolean	g_hash_table_contains		(GHashTable *table,
						 gconstpointer key);

guint		camel_folder_summary_get_deleted_count
						(CamelFolderSummary *summary);
guint		camel_folder_summary_get_junk_count
						(CamelFolderSummary *summary);
guint		camel_folder_summary_get_unread_count
						(CamelFolderSummary *summary);
#define camel_folder_summary_get_array camel_folder_summary_array
#define camel_folder_summary_get camel_folder_summary_uid
void		camel_folder_summary_free_array	(GPtrArray *array);

#define CamelSubscribable CamelStore
#define CAMEL_SUBSCRIBABLE CAMEL_STORE
#define camel_subscribable_folder_subscribed camel_store_folder_subscribed
#define camel_subscribable_folder_unsubscribed camel_store_folder_unsubscribed

#define CAMEL_FOLDER_TYPE_CONTACTS (0)
#define CAMEL_FOLDER_TYPE_EVENTS   (0)
#define CAMEL_FOLDER_TYPE_MEMOS    (0)
#define CAMEL_FOLDER_TYPE_TASKS    (0)

void		camel_operation_push_message	(GCancellable *cancellable,
						 const gchar *text);
void		camel_operation_pop_message	(GCancellable *cancellable);
CamelSession *	camel_service_ref_session	(CamelService *service);

typedef void	(*CamelSessionCallback)		(CamelSession *session,
						 GCancellable *cancellable,
						 gpointer user_data,
						 GError **error);

void		camel_session_submit_job	(CamelSession *session,
						 CamelSessionCallback callback,
						 gpointer user_data,
						 GDestroyNotify notify);

/* copy from GLib - begin */
typedef struct _GBytes          GBytes;

GBytes *        g_bytes_new                     (gconstpointer   data,
                                                 gsize           size);

GBytes *        g_bytes_new_take                (gpointer        data,
                                                 gsize           size);

GBytes *        g_bytes_new_static              (gconstpointer   data,
                                                 gsize           size);

GBytes *        g_bytes_new_with_free_func      (gconstpointer   data,
                                                 gsize           size,
                                                 GDestroyNotify  free_func,
                                                 gpointer        user_data);

GBytes *        g_bytes_new_from_bytes          (GBytes         *bytes,
                                                 gsize           offset,
                                                 gsize           length);

gconstpointer   g_bytes_get_data                (GBytes         *bytes,
                                                 gsize          *size);

gsize           g_bytes_get_size                (GBytes         *bytes);

GBytes *        g_bytes_ref                     (GBytes         *bytes);

void            g_bytes_unref                   (GBytes         *bytes);

gpointer        g_bytes_unref_to_data           (GBytes         *bytes,
                                                 gsize          *size);

GByteArray *    g_bytes_unref_to_array          (GBytes         *bytes);

guint           g_bytes_hash                    (gconstpointer   bytes);

gboolean        g_bytes_equal                   (gconstpointer   bytes1,
                                                 gconstpointer   bytes2);

gint            g_bytes_compare                 (gconstpointer   bytes1,
                                                 gconstpointer   bytes2);
/* copy from GLib - end */

#endif /* E_EWS_GNOME_3_8_COMPAT_H */
